Class: LMDB::Database

Inherits:
Object
  • Object
show all
Includes:
Enumerable
Defined in:
ext/lmdb_ext/lmdb_ext.c,
lib/lmdb/database.rb,
ext/lmdb_ext/lmdb_ext.c

Overview

An LMDB Database is a table of key-value pairs. It is stored as part of the Environment.

By default, each key in a Database maps to one value. However, a Database can be configured at creation to allow duplicate keys, in which case one key will map to multiple values.

A Database stores the keys in a sorted order. The order can also be set with options when the database is created.

The basic operations on a database are to #put, #get, and #delete records. One can also iterate through the records in a database using a Cursor.

Examples:

Typical usage

env = LMDB.new "databasedir"
db = env.database "databasename"
db.put "key1", "value1"
db.put "key2", "value2"
db.get "key1"              #=> "value1"
env.close

Instance Method Summary collapse

Instance Method Details

#[](key) ⇒ Object

Retrieve the value of a record from a database

Parameters:

  • key

    the record key to retrieve

Returns:

  • value of the record for that key, or nil if there is no record with that key

See Also:

  • #get(key)


46
47
48
# File 'lib/lmdb/database.rb', line 46

def [](key)
  get(key)
end

#[]=(key, value) ⇒ Object

Set (write or update) a record in a database.

Examples:

db['a'] = 'b'     #=> 'b'
db['b'] = 1234    #=> 1234
db['a']           #=> 'b'

Parameters:

  • key

    key for the record

  • value

    the value of the record

Returns:

  • returns the value of the record

See Also:

  • value)


59
60
61
62
# File 'lib/lmdb/database.rb', line 59

def []=(key, value)
  put key, value
  value
end

#cardinality(key) ⇒ Integer

Return the cardinality (number of duplicates) of a given key. Works whether :dupsort is set or not.

Parameters:

  • key (#to_s)

    The key in question.

Returns:

  • (Integer)

    The number of entries under the key.



118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
# File 'lib/lmdb/database.rb', line 118

def cardinality(key)
  ret = 0
  env.transaction? true do
    if get key
      if dupsort?
        cursor do |c|
          c.set key
          ret = c.count
        end
      else
        ret = 1
      end
    end
  end
  ret
end

#clearObject

Note:

The clear happens transactionally.

Empty out the database

Returns:

  • nil



1517
1518
1519
1520
1521
1522
1523
# File 'ext/lmdb_ext/lmdb_ext.c', line 1517

static VALUE database_clear(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self, "clear", 0, 0, 0);
  check(mdb_drop(need_txn(database->env), database->dbi, 0));
  return Qnil;
}

#cursor {|cursor| ... } ⇒ Object

Create a cursor to iterate through a database. Uses current transaction, if any. Otherwise, if called with a block, creates a new transaction for the scope of the block. Otherwise, fails.

Examples:

db = env.database "abc"
db.cursor do |c|
  key, value = c.next
  puts "#{key}: #{value}"
end

Yields:

  • (cursor)

    A block to be executed with the cursor.

Yield Parameters:

  • cursor (Cursor)

    The cursor to be used to iterate

See Also:



1744
1745
1746
1747
1748
1749
1750
1751
1752
1753
1754
1755
1756
1757
1758
1759
1760
1761
1762
1763
1764
1765
1766
1767
1768
1769
1770
1771
1772
1773
1774
1775
1776
1777
1778
1779
1780
1781
1782
1783
1784
# File 'ext/lmdb_ext/lmdb_ext.c', line 1744

static VALUE database_cursor(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env)) {
    if (!rb_block_given_p()) {
      rb_raise(cError, "Must call with block or active transaction.");
    }
    return call_with_transaction(database->env, self, "cursor", 0, 0, 0);
  }

  MDB_cursor* cur;
  check(mdb_cursor_open(need_txn(database->env), database->dbi, &cur));

  Cursor* cursor;
  VALUE vcur = TypedData_Make_Struct(cCursor, Cursor,
                                     &lmdb_cursor_type, cursor);
  cursor->cur = cur;
  cursor->db = self;

  if (rb_block_given_p()) {
    int exception;
    VALUE ret = rb_protect(rb_yield, vcur, &exception);
    if (exception) {
      cursor_close(vcur);
      rb_jump_tag(exception);
    }
    cursor_close(vcur);
    return ret;
  }
  else {
    VALUE vtxn = environment_active_txn(database->env);
    if (NIL_P(vtxn)) {
      rb_fatal("Internal error: transaction finished unexpectedly.");
    }
    else {
      TRANSACTION(vtxn, &lmdb_transaction_type, txn);
      rb_ary_push(txn->cursors, vcur);
    }
  }

  return vcur;
}

#delete(key, value = nil) ⇒ Object

Deletes records from the database. This function removes key/data pairs from the database. If the database does not support sorted duplicate data items (:dupsort) the value parameter is ignored. If the database supports sorted duplicates and the value parameter is nil, all of the duplicate data items for the key will be deleted. Otherwise, if the data parameter is non-nil only the matching data item will be deleted.

Parameters:

  • key

    The key of the record to delete.

  • value

    The optional value of the record to delete.

Raises:

  • (Error)

    if the specified key/value pair is not in the database.



1633
1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
1644
1645
1646
1647
1648
1649
1650
1651
1652
1653
1654
1655
1656
1657
1658
# File 'ext/lmdb_ext/lmdb_ext.c', line 1633

static VALUE database_delete(int argc, VALUE *argv, VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self, "delete", argc, argv, 0);

  VALUE vkey, vval;
  rb_scan_args(argc, argv, "11", &vkey, &vval);

  vkey = StringValue(vkey);

  MDB_val key;
  key.mv_size = RSTRING_LEN(vkey);
  key.mv_data = RSTRING_PTR(vkey);

  if (NIL_P(vval)) {
    check(mdb_del(need_txn(database->env), database->dbi, &key, 0));
  } else {
    vval = StringValue(vval);
    MDB_val value;
    value.mv_size = RSTRING_LEN(vval);
    value.mv_data = RSTRING_PTR(vval);
    check(mdb_del(need_txn(database->env), database->dbi, &key, &value));
  }

  return Qnil;
}

#delete?(key, value = nil) ⇒ void

This method returns an undefined value.

Delete the key (and optional value pair) if it exists; do not complain about missing keys.

Parameters:

  • key (#to_s)

    The key of the record

  • value (#to_s, nil) (defaults to: nil)

    The optional value

See Also:



185
186
187
# File 'lib/lmdb/database.rb', line 185

def delete?(key, value = nil)
  env.transaction? { |txn| delete(key, value) if has?(key, value) }
end

#dropObject

Note:

The drop happens transactionally.

Remove a database from the environment.

Returns:

  • nil



1503
1504
1505
1506
1507
1508
1509
# File 'ext/lmdb_ext/lmdb_ext.c', line 1503

static VALUE database_drop(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self, "drop", 0, 0, 0);
  check(mdb_drop(need_txn(database->env), database->dbi, 1));
  return Qnil;
}

#dupfixed?true, false

Returns whether the database is in :dupfixed mode.

Returns:

  • (true, false)


1486
1487
1488
1489
1490
1491
1492
1493
1494
1495
# File 'ext/lmdb_ext/lmdb_ext.c', line 1486

static VALUE database_is_dupfixed(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self,
                                 "dupfixed?", 0, 0, MDB_RDONLY);
  unsigned int flags;
  check(mdb_dbi_flags(need_txn(database->env), database->dbi, &flags));

  return (flags & MDB_DUPFIXED) == 0 ? Qfalse : Qtrue;
}

#dupsort?true, false

Returns whether the database is in :dupsort mode.

Returns:

  • (true, false)


1470
1471
1472
1473
1474
1475
1476
1477
1478
1479
# File 'ext/lmdb_ext/lmdb_ext.c', line 1470

static VALUE database_is_dupsort(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self,
                                 "dupsort?", 0, 0, MDB_RDONLY);
  unsigned int flags;
  check(mdb_dbi_flags(need_txn(database->env), database->dbi, &flags));

  return (flags & MDB_DUPSORT) == 0 ? Qfalse : Qtrue;
}

#each {|i| ... } ⇒ Object

Iterate through the records in a database

Examples:

db.each do |record|
  key, value = record
  puts "at #{key}: #{value}"
end

Yields:

  • (i)

    Gives a record [key, value] to the block

Yield Parameters:

  • i (Array)

    The key, value pair for each record



31
32
33
34
35
36
37
38
39
# File 'lib/lmdb/database.rb', line 31

def each &block
  env.transaction? true do
    cursor do |c|
      while i = c.next
        yield(i)
      end
    end
  end
end

#each_key(&block) ⇒ Enumerator

Iterate over each key in the database, skipping over duplicate records.

Returns:

  • (Enumerator)

    in lieu of a block.



74
75
76
77
78
79
80
81
82
83
84
# File 'lib/lmdb/database.rb', line 74

def each_key(&block)
  return enum_for :each_key unless block

  env.transaction? true do
    cursor do |c|
      while (rec = c.next true)
        block.call rec.first
      end
    end
  end
end

#each_value(key, &block) ⇒ Enumerator

Iterate over the duplicate values of a given key, using an implicit cursor. Works whether :dupsort is set or not.

Parameters:

  • key (#to_s)

    The key in question.

Returns:

  • (Enumerator)

    in lieu of a block.



92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
# File 'lib/lmdb/database.rb', line 92

def each_value(key, &block)
  return enum_for :each_value, key unless block

  op = -> txn do
    value = get(key) or return
    unless dupsort?
      block.call value
      return
    end

    cursor do |c|
      rec = c.set key
      while rec
        block.call rec.last
        rec = c.next_range key
      end
    end
  end

  env.transaction? true, &op
end

#empty?Boolean

Returns whether the database is empty.

Returns:

  • (Boolean)

    whether the database is empty



198
199
200
# File 'lib/lmdb/database.rb', line 198

def empty?
  stat[:entries] == 0
end

#envEnvironment

Returns the environment to which this database belongs.

Returns:

  • (Environment)

    the environment to which this database belongs.



1790
1791
1792
1793
# File 'ext/lmdb_ext/lmdb_ext.c', line 1790

static VALUE database_env(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  return database->env;
}

#flagsHash

Return the flags used to open the database.

Returns:

  • (Hash)

    The flags.



1453
1454
1455
1456
1457
1458
1459
1460
1461
# File 'ext/lmdb_ext/lmdb_ext.c', line 1453

static VALUE database_get_flags(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env,
                                 self, "flags", 0, 0, MDB_RDONLY);
  unsigned int flags;
  check(mdb_dbi_flags(need_txn(database->env), database->dbi, &flags));
  return flags2hash(flags);
}

#get(key) ⇒ Object

Retrieves one value associated with this key. This function retrieves key/data pairs from the database. If the database supports duplicate keys (:dupsort) then the first data item for the key will be returned. Retrieval of other items requires the use of #cursor.

Parameters:

  • key

    The key of the record to retrieve.



1534
1535
1536
1537
1538
1539
1540
1541
1542
1543
1544
1545
1546
1547
1548
1549
1550
1551
1552
# File 'ext/lmdb_ext/lmdb_ext.c', line 1534

static VALUE database_get(VALUE self, VALUE vkey) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self, "get", 1, &vkey, MDB_RDONLY);


  vkey = StringValue(vkey);
  MDB_val key, value;
  key.mv_size = RSTRING_LEN(vkey);
  key.mv_data = RSTRING_PTR(vkey);

  // rb_warn("lol database get %s", (char *)key.mv_data);

  int ret = mdb_get(need_txn(database->env), database->dbi, &key, &value);
  if (ret == MDB_NOTFOUND)
    return Qnil;
  check(ret);
  return rb_str_new(value.mv_data, value.mv_size);
}

#has?(key, value = nil) ⇒ Boolean

Test if the database has a given key (or, if opened in :dupsort, value)

Returns:

  • (Boolean)


137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/lmdb/database.rb', line 137

def has?(key, value = nil)
  env.transaction? true do
    if v = get(key)
      if value.nil? or value.to_s == v
        true
      elsif !dupsort?
        false
      else
        ret = false
        cursor { |c| ret = !!c.set(key, value) }

        ret
      end
    end
  end
end

#keysArray

Get the keys as an array.

Returns:

  • (Array)

    of keys.



66
67
68
# File 'lib/lmdb/database.rb', line 66

def keys
  each_key.to_a
end

#put(key, value, options) ⇒ Object

Stores items into a database. This function stores key/value pairs in the database. The default behavior is to enter the new key/value pair, replacing any previously existing key if duplicates are disallowed, or adding a duplicate data item if duplicates are allowed (:dupsort).

Parameters:

  • key

    The key of the record to set

  • value

    The value to insert for this key

Options Hash (options):

  • :nodupdata (Boolean)

    Enter the new key/value pair only if it does not already appear in the database. This flag may only be specified if the database was opened with :dupsort. The function will raise an Error if the key/data pair already appears in the database.

  • :nooverwrite (Boolean)

    Enter the new key/value pair only if the key does not already appear in the database. The function will raise an {Error] if the key already appears in the database, even if the database supports duplicates (:dupsort).

  • :append (Boolean)

    Append the given key/data pair to the end of the database. No key comparisons are performed. This option allows fast bulk loading when keys are already known to be in the correct order. Loading unsorted keys with this flag will cause data corruption.

  • :appenddup (Boolean)

    As above, but for sorted dup data.



1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
1598
1599
1600
1601
1602
1603
1604
1605
1606
1607
1608
1609
1610
1611
1612
1613
1614
1615
1616
# File 'ext/lmdb_ext/lmdb_ext.c', line 1587

static VALUE database_put(int argc, VALUE *argv, VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env, self, "put", argc, argv, 0);

  VALUE vkey, vval, option_hash = Qnil;
#ifdef RB_SCAN_ARGS_KEYWORDS
  rb_scan_args_kw(RB_SCAN_ARGS_LAST_HASH_KEYWORDS,
                  argc, argv, "20:", &vkey, &vval, &option_hash);
#else
  rb_scan_args(argc, argv, "20:", &vkey, &vval, &option_hash);
#endif

  int flags = 0;
  if (!NIL_P(option_hash))
    rb_hash_foreach(option_hash, (int (*)(ANYARGS))database_put_flags,
                    (VALUE)&flags);

  vkey = StringValue(vkey);
  vval = StringValue(vval);

  MDB_val key, value;
  key.mv_size = RSTRING_LEN(vkey);
  key.mv_data = RSTRING_PTR(vkey);
  value.mv_size = RSTRING_LEN(vval);
  value.mv_data = RSTRING_PTR(vval);

  check(mdb_put(need_txn(database->env), database->dbi, &key, &value, flags));
  return Qnil;
}

#put?(key, value = nil, **options) ⇒ void

This method returns an undefined value.

Conditionally put a value into the database.

Parameters:

  • key (#to_s)

    The key of the record

  • value (#to_s, nil) (defaults to: nil)

    the (optional) value

  • options (Hash)

    options

See Also:



164
165
166
167
168
169
170
171
172
173
# File 'lib/lmdb/database.rb', line 164

def put?(key, value = nil, **options)
  # early bailout
  return if value.nil?

  flags = { (dupsort? ? :nodupdata : :nooverwrite) => true }

  env.transaction? do |txn|
    put(key, value, **options.merge(flags)) unless has?(key, value)
  end
end

#sizeObject

Return how many records there are in this database.

Returns:

  • the number of records in this database



192
193
194
# File 'lib/lmdb/database.rb', line 192

def size
  stat[:entries]
end

#statHash

Return useful statistics about a database.

  • :psize Size of a database page

  • :depth Depth (height) of the B-tree

  • :branch_pages Number of internal (non-leaf) pages

  • :leaf_pages Number of leaf pages

  • :overflow_pages Number of overflow pages

  • :entries Number of data items

Returns:

  • (Hash)

    the statistics



1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
# File 'ext/lmdb_ext/lmdb_ext.c', line 1437

static VALUE database_stat(VALUE self) {
  DATABASE(self, &lmdb_database_type, database);
  if (!active_txn(database->env))
    return call_with_transaction(database->env,
                                 self, "stat", 0, 0, MDB_RDONLY);

  MDB_stat stat;
  check(mdb_stat(need_txn(database->env), database->dbi, &stat));
  return stat2hash(&stat);
}